"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.DownloadTask = void 0;
const storage_1 = require("@google-cloud/storage");
const aws_sdk_1 = require("aws-sdk");
const https_1 = require("https");
const path_1 = __importDefault(require("path"));
const serverCredentials_1 = require("../../../database/entities/serverCredentials");
const credentialsType_1 = require("../../../database/entities/types/credentialsType");
const agent = new https_1.Agent({ rejectUnauthorized: false });
class DownloadTask {
    constructor(symbolServer, symbolName, debugId, template, cancellationToken, _waitList, savePath = '') {
        var _a;
        this.symbolServer = symbolServer;
        this.symbolName = symbolName;
        this.debugId = debugId;
        this.template = template;
        this.cancellationToken = cancellationToken;
        this._waitList = _waitList;
        this.savePath = savePath;
        this.startDate = new Date();
        this.numberOfRetry = 0;
        const originalSymbolName = symbolName;
        const originalDebugId = debugId;
        if (symbolServer.serverCredentials) {
            const credentials = symbolServer.serverCredentials;
            this.symbolName = this.modifySymbolByType(this.symbolName, credentials.lowerFile);
            this.debugId = this.modifySymbolByType(this.debugId, credentials.lowerId);
        }
        this.symbolName = encodeURI(this.symbolName.replace(/ /g, '+'));
        this.debugId = encodeURI(this.debugId.replace(/ /g, '+'));
        this.query = this.generateRequestUrl({
            baseName: path_1.default.basename(this.symbolName, path_1.default.extname(this.symbolName)),
            symbolName: this.symbolName,
            extensionName: path_1.default.extname(this.symbolName),
            underscoreExtensionName: this.symbolName.substr(0, this.symbolName.length - 1) + '_',
            debugId: this.debugId,
            bucketName: (_a = symbolServer.serverCredentials) === null || _a === void 0 ? void 0 : _a.bucketName,
            filePrefix: this.symbolName.substr(0, 2)
        });
        this.downloadUrl = `${this.symbolServer.url}/${this.query}`;
        this._fullSymbolPath = path_1.default.join(this.savePath, originalSymbolName, originalDebugId, `${path_1.default.basename(originalSymbolName, path_1.default.extname(originalSymbolName))}${path_1.default.extname(this.downloadUrl)}`);
        if (!this.symbolServer.retryLimit) {
            this.symbolServer.retryLimit = 3;
        }
    }
    get retryTimeout() {
        return this.symbolServer.retryTimeout || 3000;
    }
    get waitList() {
        return this._waitList;
    }
    get fullSymboLPath() {
        return this._fullSymbolPath;
    }
    async authDownloadUrl() {
        var _a;
        if (this._authUrl) {
            return this._authUrl;
        }
        const serverType = (_a = this.symbolServer.serverCredentials) === null || _a === void 0 ? void 0 : _a.type;
        const credentials = this.symbolServer.serverCredentials;
        switch (serverType) {
            case credentialsType_1.CredentialsType.AWS:
                const s3 = new aws_sdk_1.S3({
                    endpoint: this.symbolServer.url,
                    accessKeyId: credentials === null || credentials === void 0 ? void 0 : credentials.accessKeyId,
                    secretAccessKey: credentials === null || credentials === void 0 ? void 0 : credentials.secretAccessKey,
                    signatureVersion: 'v4',
                });
                await new Promise((res) => {
                    s3.headObject({
                        Bucket: credentials === null || credentials === void 0 ? void 0 : credentials.bucketName,
                        Key: this.query,
                    }, (err) => {
                        if (err) {
                            res(false);
                        }
                        else {
                            res(true);
                        }
                    });
                });
                return await new Promise((res, rej) => {
                    s3.getSignedUrl('getObject', {
                        Bucket: credentials === null || credentials === void 0 ? void 0 : credentials.bucketName,
                        Key: this.query,
                        Expires: 900,
                    }, (err, signedUrl) => {
                        if (err) {
                            return rej(err);
                        }
                        this._authUrl = signedUrl;
                        return res(signedUrl);
                    });
                });
            case credentialsType_1.CredentialsType.GCP:
                const storage = new storage_1.Storage({
                    keyFilename: credentials === null || credentials === void 0 ? void 0 : credentials.secretFile,
                });
                const bucket = storage.bucket(credentials === null || credentials === void 0 ? void 0 : credentials.bucketName);
                const expirationDate = new Date(Date.now() + 300000);
                const result = await bucket.file(this.query).getSignedUrl({
                    action: 'read',
                    expires: expirationDate.toISOString(),
                });
                this._authUrl = result[0];
                return this._authUrl;
            case credentialsType_1.CredentialsType.Default:
            default:
                this._authUrl = this.downloadUrl;
                return this.downloadUrl;
        }
    }
    removeFromWaitList() {
        this._waitList = false;
    }
    hasExceededMaximumRetries() {
        return this.numberOfRetry === this.symbolServer.retryLimit;
    }
    cancelRequests() {
        this.cancellationToken.cancel(`${this.downloadUrl} returned valid symbol from symbolServer`);
    }
    getAxiosConfiguration() {
        return {
            responseType: 'stream',
            auth: this.getAuth(),
            cancelToken: this.cancellationToken.token,
            timeout: this.symbolServer.timeout || 100000,
            httpsAgent: agent,
            proxy: this.getProxy(),
            validateStatus: (status) => {
                return status === 200;
            },
        };
    }
    modifySymbolByType(symbol, type) {
        if (!type) {
            return symbol;
        }
        switch (type) {
            case serverCredentials_1.SymbolNameType.LowerCase: {
                return symbol.toLowerCase();
            }
            case serverCredentials_1.SymbolNameType.UpperCase: {
                return symbol.toUpperCase();
            }
        }
    }
    getProxy() {
        const proxy = this.symbolServer.proxy;
        return proxy
            ? {
                host: proxy.host,
                port: proxy.port,
                auth: {
                    password: proxy.password,
                    username: proxy.username,
                },
            }
            : false;
    }
    getAuth() {
        const auth = this.symbolServer.serverCredentials;
        return auth && auth.type === credentialsType_1.CredentialsType.Default && auth.userName && auth.password
            ? {
                username: auth.userName,
                password: auth.password,
            }
            : undefined;
    }
    generateRequestUrl(opts) {
        let query = this.template;
        for (const key in opts) {
            if (opts.hasOwnProperty(key)) {
                const element = opts[key];
                const regex = new RegExp(`{${key}}`, 'g');
                query = query.replace(regex, element);
            }
        }
        return query;
    }
}
exports.DownloadTask = DownloadTask;
//# sourceMappingURL=downloadTask.js.map